# -*- coding: utf-8 -*-

# Standard Library
import logging

# Vendor Library
from celery.result import states
from m3.actions import ApplicationLogicException

# Project Library
from kinder.core.async.tasks import AsyncTask
from kinder.core.unit.models import Unit, UnitKind
from kinder.core.helpers import get_current_user
from gisdo import settings as gs_settings
from gisdo.exceptions import (
    ReportGenerationException)
from gisdo.models import (
    ReportForm, ReportFormRow, GisdoUnit)
from gisdo.reports import ReportFormReport
from gisdo.service import ReportService


log = logging.getLogger('gisdo')


def _get_report_progress(current_unit_num, units_count):
        """
        Подсчет прогресса выполнения
        @current_unit_num - порядковый номер текущего учреждения
        @units_count - количество учреждений
        """

        return int(100*float(current_unit_num)/float(units_count))


class Report(AsyncTask):
    """
    Отчет по собранным показателям
    """

    description = u'Отчет по собранным показателям'
    stop_executing = False
    state = {
        'values': {}, 'description': u'Неизвестно',
        'progress': u'Неизвестно'}

    def run(self, result_name, report_id, full_name, email, phone):
        """Выполнение задачи"""

        self.set_progress(
            u'Формирование отчета...', task_state=u'RUNNING')

        self.result_name = result_name
        report = ReportFormReport(result_name)

        report.flush_header(full_name, email, phone)

        try:
            report_form = ReportForm.objects.get(id=report_id)
        except ReportForm.DoesNotExist:
            raise ReportGenerationException(
                u'Отсутствуют данные по учреждению')

        report_form_rows = ReportFormRow.objects.filter(
            report=report_form)
        rows_count = report_form_rows.count()

        row_number = 0
        row_data = []
        for report_form_row in report_form_rows:
            try:
                not_on_federal_report = (
                    report_form_row.unit.gisdo.not_on_federal_report)
            except GisdoUnit.DoesNotExist:
                raise ApplicationLogicException(
                    u'Учреждение c ID=%d не найдено'
                    u' в модуле "Фед. отчетности"' %
                    report_form_row.unit.pk)
            if (
                report_form_row.unit.kind.id == UnitKind.DOU and
                    not not_on_federal_report):

                self.set_progress(
                    u'Пeчать показателей по %s. (%s%%)' % (
                        report_form_row.unit.display(),
                        _get_report_progress(
                            row_number, rows_count)))

                parent_unit = report_form_row.unit.get_mo(
                ) or report_form.unit
                ocato = parent_unit.ocato if parent_unit else ''
                try:
                    row_data.append(
                        report.collect_row_data(
                            report_form_row, report_form.date, ocato))
                except (KeyError, UnicodeDecodeError):
                    err_msg = (
                        u'При печате показателей по '
                        u'%s произошла ошибка' %
                        report_form_row.unit.display())
                    raise KeyError(err_msg)

                row_number += 1

        report.flush_row_data(sorted(row_data, key=lambda x: x['mo']))
        report.build()

        return self.state

    def on_success(self, retval, task_id, args, kwargs):

        url = '%s/%s.xlsx' % (gs_settings.DOWNLOADS_URL, self.result_name)
        self.set_progress(
            u'<a href="%s" target="_blank">'
            u'Отчет сформирован</a>' % url,
            task_state=states.SUCCESS)

    def on_failure(self, exc, task_id, args, kwargs, einfo):

        self.set_progress(exc.message, task_state=states.FAILURE)


class CollectUnitsDataTask(AsyncTask):
    """
    Сбор и сохранение данных по учреждениям
    """
    description = u'Подсчет показателей'
    stop_executing = False
    state = {
        'values': {}, 'description': u'Неизвестно',
        'progress': u'Неизвестно'}

    def run(self, context, **kwargs):
        """Выполнение задачи"""
        self.set_progress(u'Подсчет показателей...', task_state=u'RUNNING')

        ReportService(
            context.unit,
            self.set_progress,
            user=context.user,
        ).collect(send=False)

        return self._result_message(u'Формирование завершено', state=states.SUCCESS)


class CollectUnitDataPeriodicTask(CollectUnitsDataTask):
    """
    Подсчет и отправка показателей (для периодического выполнения с celery beat)
    """
    description = u'Подсчет и отправка показателей'

    def run(self, **kwargs):
        """Выполнение задачи"""
        self.set_progress(u'Подсчет показателей...', task_state=u'RUNNING')

        user = get_current_user()

        highest_units = Unit.objects.filter(parent__isnull=True)
        if highest_units.count() == 1:
            unit = highest_units[0]
        else:
            raise ApplicationLogicException(
                u'В корне дерева учреждений должен быть только один элемент')

        ReportService(
            unit,
            self.set_progress,
            user=user,
        ).collect(send=True)

        return self._result_message(u'Формирование завершено', state=states.SUCCESS)

    def on_failure(self, exc, task_id, args, kwargs, einfo):

        try:
            msg = unicode(exc.message)
        except UnicodeDecodeError:
            msg = exc.message.decode('UTF-8')

        self.set_progress(msg, task_state=states.FAILURE)